#!/usr/bin/env Rscript

options(max.print=1000000) 

source("sqlconfig.R")
#tbl <- paste("results_code_1to5faults_check")
tbl <- paste("results_code32_1to8faults_check")

con = sql_connect(tbl)
on.exit(dbDisconnect(con))

# -- Get table information
query <- paste(c("describe ", tbl, ";"), collapse='')

rs <- dbSendQuery(con, query)
data <- fetch(rs)
#(data)
# Number of *sdc* columns in tbl
numFaults <- (length(grep("sdc", data[,1])))
dbClearResult(rs)

# -- Generate query according to number of bit faults.
#query <-  paste(c("SELECT A, (p1_sdc/p1_sum) AS p1, (p2_sdc/p2_sum) AS p2, (p3_sdc/p3_sum) AS p3 FROM ", sqldbname, ".", tbl, ";"),collapse=''
query <-  paste("SELECT A")
for(i in 1:numFaults) {
  query <- (paste(c(query, ",round((p", i, "_sdc/p", i, "_sum),16) AS p", i), collapse=''))
}
query <- paste(c(query, " FROM ", tbl, ";"), collapse='')
(query) # print query for debugging

rs <- dbSendQuery(con, query)
data <- fetch(rs, -1)
(data)

# -- Plot graph

library (reshape)
library (ggplot2)
filename <- paste(c("gen/", tbl, ".png"), collapse='')

attach(data)
df <- data.frame(A, p1, p2, p3)
colnames(df) <- c("A", "Bit Flips", "Psdc")
#(df)
d <- ggplot(df, aes(x=A, y=Psdc, color="Bit Flips"))
d <- d + scale_colour_brewer(palette="Spectral")
	d <- d + geom_point(aes(y=p1,  col="01 bit"), size=0.9)
	d <- d + geom_point(aes(y=p2,  col="02 bit"), size=0.9)
	d <- d + geom_point(aes(y=p3,  col="03 bit"), size=0.9)
	d <- d + geom_point(aes(y=p4,  col="04 bit"), size=0.9)
	d <- d + geom_point(aes(y=p5,  col="05 bit"), size=0.9)
	d <- d + geom_point(aes(y=p6,  col="06 bit"), size=0.9)
	d <- d + geom_point(aes(y=p7,  col="07 bit"), size=0.9)
	d <- d + geom_point(aes(y=p8,  col="08 bit"), size=0.9)
d <- d + scale_y_log10() + xlab("A") + ylab(expression("Residual Error Probability " * p[sdc]))
d <- d + ggtitle(paste(c("Simulated Residual Error Probability (All As: 1 - 65536, 1 to 8-bit flips)"), collapse=''))


ggsave(filename, width=20, height=10, dpi=200)
detach(data)



