#include "db_connection.h"
#include <mysql.h>
#include <stdio.h>
#include "config.h"
#include <stdlib.h>

const char *server   = "faui42.informatik.uni-erlangen.de";
const char *user     = "cored";
const char *password = "iq80310";
const char *database = "cored_hpc";

//#define DB_DBG

// Generate table name
#define STRINGIFY(x) #x
#define EXPAND(str) STRINGIFY(str)
const char* g_db_table = "results_code" EXPAND(__INT)
                           "_1to" EXPAND(__FAULTS) "faults_"
                           EXPAND(__CHECK)
#ifdef __TABLEPOSTFIX
                           "." EXPAND(__TABLEPOSTFIX)
#endif
                           ;


void db_testQ(int num){
    if (num >= QUERYSIZE){
        fprintf(stderr, "SQL statement too long :(\nIncrease QUERYSIZE!");
        exit(1);
    }
}


static  MYSQL * conn = 0;

// internal connect and query
int db_connect(void)
{
  conn = mysql_init(NULL);
  if(!mysql_real_connect( conn, server, user, password, database, 0, NULL, 0 )){
    fprintf(stderr, "%s\n", mysql_error(conn));
    return DBERR;
  }
  return DBOK;
}

static int db_q(char * q){
#ifdef DB_DBG
  printf("Trying to execute query: %s\n", q);
#endif
  if( conn == 0 ){
    fprintf(stderr, "Error. No connection established!\n");
    return DBERR;
  }

  if(mysql_query(conn, q)){
    fprintf(stderr, "%s\n", mysql_error(conn));
    mysql_close(conn);
    return DBERR;
  }
  return DBOK;
}

// internal close
int db_close()
{
  mysql_close(conn);
  conn = 0;
  return DBOK;
}

// a database query ignoring the affected rows and results.
int db_query(char* q)
{

  if( db_connect() != DBOK ){
    mysql_close(conn);
    return DBERR;
  }

  return db_q(q);
}


// checks if a table exists in the database
// awaits an open connection.
int db_table_exists(const char* tablename){
  char query[QUERYSIZE];

  int num = snprintf(query, QUERYSIZE, "SELECT count(*) from information_schema.tables WHERE table_schema = '%s' AND table_name = '%s'", database, g_db_table);
  db_testQ(num);

  if( db_q(query) != DBOK ) {
    return DBERR;
  }
  // check if table exists -> row = 1
  MYSQL_RES *result = mysql_store_result(conn);

  int num_fields = mysql_num_fields(result);

  int retval = 0;

  if( num_fields == 1) {
   MYSQL_ROW row;
   row = mysql_fetch_row(result);
   if(row[0][0] == '1') {
     retval = 1;
   }
  }

  mysql_free_result(result);

  return retval;
}

// number of rows in table 
// awaits an open connection.
int db_table_rowcount(const char* tablename)
{
  char query[QUERYSIZE];

  int num = snprintf(query, QUERYSIZE, "SELECT count(*) FROM %s", g_db_table);
  db_testQ(num);

  if( db_q(query) != DBOK ) {
    return DBERR;
  }
  // check if table exists -> row = 1
  MYSQL_RES *result = mysql_store_result(conn);

  int num_fields = mysql_num_fields(result);

  int retval = 0;

  if( num_fields == 1) {
   MYSQL_ROW row;
   row = mysql_fetch_row(result);
   sscanf(row[0], "%d", &retval);
  }

  mysql_free_result(result);

  return retval;
}


int db_create_table(void)
{
  char q[QUERYSIZE];
  int num = snprintf(q, QUERYSIZE, "CREATE  TABLE %s.%s (A INT NOT NULL\n", database, g_db_table);
  db_testQ(num);

  for(int i = 1; i <= __FAULTS; i++){
    num += snprintf(q+num, QUERYSIZE - num, ",p%d_sum BIGINT ZEROFILL NULL\n"
                                            ",p%d_due BIGINT ZEROFILL NULL\n"
                                            ",p%d_sdc BIGINT ZEROFILL NULL\n"
                                            ",p%d_ovf BIGINT ZEROFILL NULL\n"
                                            ",p%d_patterns INT ZEROFILL NULL\n",i,i,i,i,i);
    db_testQ(num);
  }

  num += snprintf(q+num, QUERYSIZE, ",PRIMARY KEY (A), UNIQUE INDEX A_UNIQUE (A ASC) )");
  db_testQ(num);

  return db_q(q);

}


int db_send_data(result_t * results){
  const A_t theA = __A;
  // REPLACE INTO g_db_table (A, p1_sum, p1_due, p1_sdc, p1_ovf, p1_patterns, p2_sum, ...) VALUES(1, 2, 33, 4);
  char q[QUERYSIZE];
  int num = snprintf(q, QUERYSIZE, "REPLACE INTO %s (A", g_db_table);
  db_testQ(num);

  for(int i = 1; i <= __FAULTS; i++){
    num += snprintf(q+num, QUERYSIZE - num, ",p%d_sum,p%d_due,p%d_sdc,p%d_ovf,p%d_patterns", i,i,i,i,i);
    db_testQ(num);
  }

  num += snprintf(q+num, QUERYSIZE - num, ") VALUES(" PRINTPLN ,theA);
  db_testQ(num);

  for(int i = 0; i < __FAULTS; i++){
    num += snprintf(q+num, QUERYSIZE - num, "," PRINTEXP "," PRINTEXP "," PRINTEXP "," PRINTEXP "," PRINTEXP,
        results[i].sum, results[i].due, results[i].sdc, results[i].ovf, results[i].patterns);
    db_testQ(num);
  }

  num += snprintf(q+num, QUERYSIZE - num, ")");
  db_testQ(num);

  return db_query(q);
}

#if 0
// DELIMITER seems not to work correctly here.
int db_create_trigger(void)
{
  char q[QUERYSIZE];
  int num = snprintf(q, QUERYSIZE, "DELIMITER |\nCREATE TRIGGER trig_%s\nBEFORE INSERT ON %s\nFOR EACH ROW\nBEGIN\n", g_db_table, g_db_table);

  for(int i = 1; i <= __FAULTS; i++) {
    num += snprintf(q+num, QUERYSIZE -num,  "SET NEW.p%d_psdc = ISNULL((NEW.p%d_sdc / NULLIF(0, NEW.p%d_sum), 0));", i, i, i);

  }

  num += snprintf(q+num, QUERYSIZE - num, "\nEND");
  puts(q);
  return db_q(q);
}
#endif
