#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <assert.h>
#include <string.h>
#include "config.h"

#include "stdout_connection.h"

#include "db_connection.h"

static experiments_t patterns = 0;

static const enc_t vc_max = (__A * __MAX_PLAIN);

// AN encode operation
static enc_t encode(plain_t v, const A_t A) {
	return v * A;
}

// AN decode operation
static plain_t decode(enc_t vc, const A_t A) {
	return vc / A;
}
// The simulation results
static result_t results[__FAULTS];

// Send data to stdout or sql database
static void send_data(){
    db_send_data(results);
    stdout_send_data(results);
}

static void testRun(enc_t faultPattern, uint8_t bitFlips) {
	enc_t due = 0;
	enc_t sdc = 0;
	enc_t over = 0;
  const A_t theA = __A;

	#ifdef __DEBUG
	assert(__POPCOUNT(faultPattern) == bitFlips);
	#endif

	for (enc_t v = 0; v <= __MAX_PLAIN; v++) {

		enc_t vc = encode(v, theA);
		enc_t vc_err = (vc ^ faultPattern); // inject pattern into encoded value

		// Remainder != 0, error detected -> DUE
		if ((vc_err % theA) != 0) {
			due++;
			continue;
		}
		#ifdef __DOCHECK
		// Value is not within defined range -> OVERFLOW
		if (vc_err > vc_max) {
 			//printf("Original: %d ["INTTOBINARYPATTERN"], Verfälscht: %d["INTTOBINARYPATTERN"], Bitmuster: ["INTTOBINARYPATTERN"]\n", werte[i], INTTOBINARY(werte[i]), vc, INTTOBINARY(vc), INTTOBINARY(bitmuster));
 			over++;
 			continue;
 		}
 		#endif

		// seems OK until now try to -> Decode
		plain_t v_dec = decode(vc_err, theA);

		// silently corrupted -> SDC
		if (v != v_dec) {
			sdc++;
		}
	}

  results[bitFlips-1].sum += __MAX_PLAIN + 1;
  results[bitFlips-1].due += due;
  results[bitFlips-1].sdc += sdc;
  results[bitFlips-1].ovf += over;
  results[bitFlips-1].patterns += 1;
}

static void generatePattern(enc_t bitPattern, uint8_t currentBit, uint8_t bitsLeft) {
	//printf("Aufruf mit: "INTTOBINARYPATTERN", CB: %u, BL: %d\n", INTTOBINARY(bitPattern), currentBit, bitsLeft);
	int16_t bit;

  for (bit = currentBit; bit > 0; bit--) {
		enc_t faultPattern = bitPattern | (1UL << (bit-1));
		patterns++;

		//printf(INTTOBINARYPATTERN" "INTTOBINARYPATTERN" %d,%d\n", INTTOBINARY(bitPattern), INTTOBINARY(faultPattern), currentBit, bit-1);
		testRun(faultPattern, __FAULTS - bitsLeft + 1);

		// Rekursion
		if ((bitsLeft > 1) && (bit > 0)) generatePattern(faultPattern, bit - 1, bitsLeft - 1);
	}
}


int main(int argc, char* argv[])
{
	// clear result array
	memset(results, 0, sizeof(results));

  //generate error pattern and start simulation
	generatePattern(0x0, __INT, __FAULTS);

  // send back results
  send_data();
}

